;=======================================================================================================
;=                                                                                                     =
;= (DWORD)CG_Main(DWORD iOption,                                                                       =
;=                DWORD *pBuffer,                                                                      =
;=                DWORD RegSrc,                                                                        =
;=                DWORD RegDest,                                                                       =
;=                DWORD GarbageReg,                                                                    =
;=                DWORD imm32,                                                                         =
;=                DWORD *pRNG,                                                                         =
;=                DWORD *piSeed,                                                                       =
;=                DWORD iPrimeNumber1,                                                                 =
;=                DWORD iPrimeNumber2                                                                  =
;=               )                                                                                     =
;=                                                                                                     =
;= Paramtres:                                                                                         =
;= -----------                                                                                         =
;=                                                                                                     =
;= DWORD iOptions      --> indique l'instruction  gnrer                                             =
;= DWORD *pBuffer      --> pointeur sur le buffer o placer l'instruction gnre                      =
;= DWORD RegSrc        --> registre source (si prsent, 90909090h sinon)                               =
;= DWORD RegDest       --> registre destination (si prsent, 90909090h sinon)                          =
;= DWORD GarbageReg    --> registre destructible (obligatoire)                                         =
;= DWORD imm32         --> imm32 (si prsent, 90909090h sinon)                                         =
;= DWORD *pRNG         --> pointeur sur le gnrateur de nombres alatoires                            =
;= DWORD *piSeed       --> pointeur sur une graine                                                     =
;= DWORD iPrimeNumber1 --> grand nombre premier                                                        =
;= DWORD iPrimeNumber2 --> grand nombre premier (<iPrimeNumber1)                                       =
;=                                                                                                     =
;= Retour:                                                                                             =
;= -------                                                                                             =
;=                                                                                                     =
;= eax == CG_ERROR si erreur                                                                           =
;= eax == CG_SUCCESS si pas d'erreurs                                                                  =
;=                                                                                                     =
;= Remarques:                                                                                          =
;= ----------                                                                                          =
;=                                                                                                     =
;= iOptions peut prendre les valeurs suivantes:                                                        =
;=                                                                                                     =
;=              a. 00h ==> mov reg, imm32                                                              =
;=              b. 01h ==> push reg                                                                    =
;=              e. 02h ==> dec reg                                                                     =
;=              f. 03h ==> inc reg                                                                     =
;=              g. 04h ==> test reg, reg (GarbageReg = reg !!!)                                        =
;=              i. 05h ==> pop reg                                                                     =
;=              j. 06h ==> jmp Xxx (Imm32 = Xxx)                                                       =
;=              k. 07h ==> add esp, 8                                                                  =
;=                                                                                                     =
;= Pile:                                                                                               =
;= -----                                                                                               =
;=                                                                                                     =
;= ESP --> _edi          -20                                                                           =
;=         _esi          -1C                                                                           =
;=         _ebp          -18                                                                           =
;=         _esp          -14                                                                           =
;=         _ebx          -10                                                                           =
;=         _edx          -0C                                                                           =
;=         _ecx          -08                                                                           =
;=         _eax          -04                                                                           =
;= EBP --> ebp           +00                                                                           =
;=         @ de retour   +04                                                                           =
;=         iOption       +08                                                                           =
;=         pBuffer       +0C                                                                           =
;=         RegSrc        +10                                                                           =
;=         RegDest       +14                                                                           =
;=         GarbageReg    +18                                                                           =
;=         Imm32         +1C                                                                           =
;=         pRNG          +20                                                                           =
;=         piSeed        +24                                                                           =
;=         iPrimeNumber1 +28                                                                           =
;=         iPrimeNumber2 +2C                                                                           =
;=                                                                                                     =
;=======================================================================================================

include CgHeader.inc
include InstrMaker.inc

_edi          = -20h
_esi          = -1Ch
_ebp          = -18h
_esp          = -14h
_ebx          = -10h
_edx          = -0Ch
_ecx          = -08h
_eax          = -04h
iOption       = +08h
pBuffer       = +0Ch
RegSrc        = +10h
RegDest       = +14h
GarbageReg    = +18h
Imm32         = +1Ch
pRNG          = +20h
piSeed        = +24h
iPrimeNumber1 = +28h
iPrimeNumber2 = +2Ch

CG_Main:                              ;
  push ebp                            ;
  mov ebp, esp                        ;
  pusha                               ;
  mov edi, [ebp+pBuffer]              ;
  mov eax, [ebp+iOption]              ;
  test eax, eax                       ;
  jz c_MovRegImm32                    ;
  cmp eax, 01h                        ;
  je c_PushReg                        ;
  cmp eax, 02h                        ;
  je c_DecReg                         ;
  cmp eax, 03h                        ;
  je c_IncReg                         ;
  cmp eax, 04h                        ;
  je c_TestRegReg                     ;
  cmp eax, 05h                        ;
  je c_PopReg                         ;
  cmp eax, 06h                        ;
  je c_JmpXxx                         ;
  cmp eax, 07h                        ;
  je c_AddEsp8                        ;
CodeGenerator_Error:                  ;
  mov dword ptr[ebp+_eax], CG_ERROR   ;
CodeGenerator_Success:                ;
  popa                                ;
  pop ebp                             ;
  ret 28h                             ;
c_MovRegImm32:                        ;
  call _MovRegImm32_                  ;
  jmp CodeGenerator_Success           ;
c_PushReg:                            ;
  call _PushReg_                      ;
  jmp CodeGenerator_Success           ;
c_DecReg:                             ;
  call _DecReg_                       ;
  jmp CodeGenerator_Success           ;
c_IncReg:                             ;
  call _IncReg_                       ;
  jmp CodeGenerator_Success           ;
c_TestRegReg:                         ;
  call _TestRegReg_                   ;
  jmp CodeGenerator_Success           ;
c_PopReg:                             ;
  call _PopReg_                       ;
  jmp CodeGenerator_Success           ;
c_JmpXxx:                             ;
  call _JmpXxx_                       ;
  jmp CodeGenerator_Success           ;
c_AddEsp8:                            ;
  call _AddEsp8_                      ;
  jmp CodeGenerator_Success           ;
                                      ;
;=======================================================================================================
; Gnration d'un <mov reg, imm32>
;=======================================================================================================
Cg_MovRegImm32_a:                     ;mov reg, imm32
  mov ebx, [ebp+RegDest]              ;
  mov ecx, [ebp+Imm32]                ;
  call MovRegImm32                    ;
  ret                                 ;
Cg_MovRegImm32_b:                     ;push imm32/pop reg
  mov ecx, [ebp+Imm32]                ;
  call PushImm32                      ;
  mov ebx, [ebp+RegDest]              ;
  call PopReg                         ;
  ret                                 ;
Cg_MovRegImm32_c:                     ;add imm32, _imm32/mov reg, imm32/sub reg, _imm32
  call RndCall                        ;
  push eax                            ;
  mov ecx, [ebp+Imm32]                ;
  add ecx, eax                        ;
  mov ebx, [ebp+RegDest]              ;
  call MovRegImm32                    ;
  pop ecx                             ;
  mov ebx, [ebp+RegDest]              ;
  test ebx, ebx                       ;
  jz Cg_SubEaxImm32                   ;
  call SubRegImm32                    ;
  ret                                 ;
Cg_MovRegImm32_d:                     ;add imm32, _imm32/push imm32/pop reg/sub reg, _imm32
  call RndCall                        ;
  push eax                            ;
  mov ecx, [ebp+Imm32]                ;
  add ecx, eax                        ;
  call PushImm32                      ;
  mov ebx, [ebp+RegDest]              ;
  call PopReg                         ;
  pop ecx                             ;
  mov ebx, [ebp+RegDest]              ;
  test ebx, ebx                       ;
  jz Cg_SubEaxImm32                   ;
  call SubRegImm32                    ;
  ret                                 ;
Cg_SubEaxImm32:                       ;
  call SubEaxImm32                    ;
  ret                                 ;
                                      ;
;=======================================================================================================
; Gnration d'un <push reg>
;=======================================================================================================
Cg_PushReg_a:                         ;push reg
  mov eax, [ebp+RegSrc]               ;
  call PushReg                        ;
  ret                                 ;
Cg_PushReg_b:                         ;mov [esp+4], reg/sub esp, 4
  mov al, 89h                         ;
  stosb                               ;
  mov al, 01000100b                   ;
  mov cl, [ebp+RegSrc]                ;
  shl cl, 3                           ;
  or al, cl                           ;
  stosb                               ;
  mov eax, 0EC830424h                 ;
  stosd                               ;
  mov al, 04h                         ;
  stosb                               ;
  ret                                 ;
                                      ;
;=======================================================================================================
; Gnration d'un <dec reg>
;=======================================================================================================
Cg_DecReg_a:                          ;dec reg
  mov ebx, [ebp+RegDest]              ;
  call DecReg                         ;
  ret                                 ;
Cg_DecReg_b:                          ;sub reg, 1
  mov ebx, [ebp+RegDest]              ;
  mov al, 10000011b                   ;
  stosb                               ;
  mov al, 11101000b                   ;
  or al, bl                           ;
  stosb                               ;
  xor al, al                          ;
  inc al                              ;
  stosb                               ;
  ret                                 ;
Cg_DecReg_c:                          ;add reg, imm32/add imm32, 1/sub reg, imm32
  call RndCall                        ;
  push eax                            ;
  mov ecx, eax                        ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegImm32                    ;
  pop ecx                             ;
  inc ecx                             ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegImm32                    ;
  ret                                 ;
Cg_DecReg_d:                          ;sub reg, imm32/sub imm32, 1/add reg, imm32
  call RndCall                        ;
  push eax                            ;
  mov ecx, eax                        ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegImm32                    ;
  pop ecx                             ;
  dec ecx                             ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegImm32                    ;
  ret                                 ;
Cg_DecReg_e:                          ;add reg1, reg2/dec reg1/sub reg1, reg2
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegReg                      ;
  mov ebx, [ebp+RegDest]              ;
  call DecReg                         ;
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegReg                      ;
  ret                                 ;
Cg_DecReg_f:                          ;sub reg1, reg2/dec reg1/add reg1, reg2
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegReg                      ;
  mov ebx, [ebp+RegDest]              ;
  call DecReg                         ;
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegReg                      ;
  ret                                 ;
                                      ;
;=======================================================================================================
; Gnration d'un <inc reg>
;=======================================================================================================
Cg_IncReg_a:                          ;inc reg
  mov ebx, [ebp+RegDest]              ;
  call IncReg                         ;
  ret                                 ;
Cg_IncReg_b:                          ;add reg, 1
  mov ebx, [ebp+RegDest]              ;
  mov al, 1000001b                    ;
  stosb                               ;
  mov al, 11000000b                   ;
  or al, bl                           ;
  stosb                               ;
  xor al, al                          ;
  inc al                              ;
  stosb                               ;
  ret                                 ;
Cg_IncReg_c:                          ;add reg, imm32/sub imm32, 1/sub reg, imm32
  call RndCall                        ;
  push eax                            ;
  mov ecx, eax                        ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegImm32                    ;
  pop ecx                             ;
  dec ecx                             ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegImm32                    ;
  ret                                 ;
Cg_IncReg_d:                          ;sub reg, imm32/add imm32, 1/add reg, imm32
  call RndCall                        ;
  push eax                            ;
  mov ecx, eax                        ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegImm32                    ;
  pop ecx                             ;
  inc ecx                             ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegImm32                    ;
  ret                                 ;
Cg_IncReg_e:                          ;add reg1, reg2/inc reg1/sub reg1, reg2
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegReg                      ;
  mov ebx, [ebp+RegDest]              ;
  call IncReg                         ;
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegReg                      ;
  ret                                 ;
Cg_IncReg_f:                          ;sub reg1, reg2/inc reg1/add reg1, reg2
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call SubRegReg                      ;
  mov ebx, [ebp+RegDest]              ;
  call IncReg                         ;
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, [ebp+RegDest]              ;
  call AddRegReg                      ;
  ret                                 ;
                                      ;
;=======================================================================================================
; Gnration d'un <test reg, reg>
;=======================================================================================================
Cg_TestRegReg_a:                      ;test reg, reg
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, eax                        ;
  call TestRegReg                     ;
  ret                                 ;
Cg_TestRegReg_b:                      ;or reg, reg
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, eax                        ;
  call OrRegReg                       ;
  ret                                 ;
Cg_TestRegReg_c:                      ;cmp reg, reg
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, eax                        ;
  call CmpRegReg                      ;
  ret                                 ;
Cg_TestRegReg_d:                      ;test reg, 0
  mov ebx, [ebp+GarbageReg]           ;
  cmp bl, 0                           ;
  je AlOptimised                      ;
  mov al, 11110111b                   ;
  stosb                               ;
  mov al, 11000000b                   ;
  or al, bl                           ;
  stosb                               ;
  jmp TestReg0End                     ;
AlOptimised:                          ;
  mov al, 10101001b                   ;
  stosb                               ;
TestReg0End:                          ;
  xor eax, eax                        ;
  stosd                               ;
  ret                                 ;
Cg_TestRegReg_e:                      ;cmp reg, 0
  mov ebx, [ebp+GarbageReg]           ;
  mov al, 001000001b                  ;
  stosb                               ;
  mov al, 11111000b                   ;
  or al, bl                           ;
  stosb                               ;
  xor al, al                          ;
  stosb                               ;
  ret                                 ;
                                      ;
;=======================================================================================================
; Gnration d'un <pop reg>
;=======================================================================================================
Cg_PopReg_a:                          ;pop reg
  mov ebx, [ebp+RegDest]              ;
  call PopReg                         ;
  ret                                 ;
Cg_PopReg_b:                          ;mov reg, [esp]/sub esp, 4
  mov ebx, [ebp+RegDest]              ;
  mov al, 10001011b                   ;
  stosb                               ;
  mov al, 00000100b                   ;
  rcl bl, 3                           ;
  or al, bl                           ;
  stosb                               ;
  mov al, 00100100b                   ;
  stosb                               ;
  mov ax, 0EC83h                      ;
  stosw                               ;
  mov al, 04h                         ;
  stosb                               ;
  ret                                 ;
                                      ;
;=======================================================================================================
; Gnration d'un <jmp Xxx>  [JzXxx]
;=======================================================================================================
Cg_JmpXxx_a:                          ;jmp Xxx
;  mov ecx, [ebp+Imm32]                ;
  call _Jmp                           ;
  ret                                 ;
Cg_JmpXxx_b:                          ;xor reg, reg/test reg, reg/jz Xxx
  call Cg_XorRegReg                   ;
  call Cg_TestRegReg_a                ;
  jmp Cg_Jz                           ;
Cg_JmpXxx_c:                          ;sub reg, reg/test reg, reg/jz Xxx
  call Cg_SubRegReg                   ;
  call Cg_TestRegReg_a                ;
  jmp Cg_Jz                           ;
Cg_JmpXxx_d:                          ;xor reg, reg/cmp reg, reg/jz Xxx
  call Cg_XorRegReg                   ;
  call Cg_TestRegReg_c                ;
  jmp Cg_Jz                           ;
Cg_JmpXxx_e:                          ;sub reg, reg/cmp reg, reg/jz Xxx
  call Cg_SubRegReg                   ;
  call Cg_TestRegReg_c                ;
  jmp Cg_Jz                           ;
Cg_JmpXxx_f:                          ;xor reg, reg/or reg, reg/jz Xxx
  call Cg_XorRegReg                   ;
  call Cg_TestRegReg_b                ;
  jmp Cg_Jz                           ;
Cg_JmpXxx_g:                          ;sub reg, reg/or reg, reg/jz Xxx
  call Cg_SubRegReg                   ;
  call Cg_TestRegReg_b                ;
  jmp Cg_Jz                           ;
Cg_JmpXxx_h:                          ;xor reg, reg/test reg, 0/jz Xxx
  call Cg_XorRegReg                   ;
  jmp lTestReg0                       ;
Cg_JmpXxx_i:                          ;sub reg, reg/test reg, 0/jz Xxx
  call Cg_SubRegReg                   ;
lTestReg0:                            ;
  call Cg_TestRegReg_d                ;
;  mov ecx, [ebp+Imm32]                ;
  call _Jz                            ;
  ret                                 ;
Cg_JmpXxx_j:                          ;xor reg, reg/cmp reg, 0/jz Xxx
  call Cg_XorRegReg                   ;
  jmp lCmpReg0                        ;
Cg_JmpXxx_k:                          ;sub reg, reg/cmp reg, 0/jz Xxx
  call Cg_SubRegReg                   ;
lCmpReg0:                             ;
  call Cg_TestRegReg_e                ;
Cg_Jz:                                ;
;  mov ecx, [ebp+Imm32]                ;
  call _Jz                            ;
  ret                                 ;
Cg_JmpXxx_l:                          ;xor ecx, ecx/jcxz Xxx
  mov eax, 01h                        ;
  mov ebx, eax                        ;
  call XorRegReg                      ;
  jmp Cg_Jcxz                         ;
Cg_JmpXxx_m:                          ;sub ecx, ecx/jcxz Xxx
  mov eax, 01h                        ;
  mov ebx, eax                        ;
  call SubRegReg                      ;
Cg_Jcxz:                              ;
;  mov ecx, [ebp+Imm32]                ;
  call _Jcxz                          ;
  ret                                 ;
Cg_JmpXxx_n:                          ;<math> garbage_reg, imm32/cmp reg, 0/jnzXxx
  call Cg_AddSub                      ;
  call Cg_TestRegReg_e                ;
  jmp Cg_JnzXxx                       ;
Cg_JmpXxx_o:                          ;<math> garbage_reg, imm32/test reg, 0/jnzXxx
  call Cg_AddSub                      ;
  call Cg_TestRegReg_d                ;
  jmp Cg_JnzXxx                       ;
Cg_JmpXxx_p:                          ;<math> garbage_reg, imm32/test reg, reg/jnzXxx
  call Cg_AddSub                      ;
  call Cg_TestRegReg_a                ;
  jmp Cg_JnzXxx                       ;
Cg_JmpXxx_q:                          ;<math> garbage_reg, imm32/or reg, reg/jnzXxx
  call Cg_AddSub                      ;
  call Cg_TestRegReg_b                ;
  jmp Cg_JnzXxx                       ;
Cg_JmpXxx_r:                          ;<math> garbage_reg, imm32/cmp reg, reg/jnzXxx
  call Cg_AddSub                      ;
  call Cg_TestRegReg_c                ;
  jmp Cg_JnzXxx                       ;
Cg_XorRegReg:                         ;
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, eax                        ;
  call XorRegReg                      ;
  ret                                 ;
Cg_SubRegReg:                         ;
  mov eax, [ebp+GarbageReg]           ;
  mov ebx, eax                        ;
  call SubRegReg                      ;
  ret                                 ;
Cg_JnzXxx:                            ;
  call _Jnz                           ;
  ret                                 ;
Cg_AddSub:                            ;
  call RndCall                        ;
  test eax, eax                       ;
  jz Cg_Add                           ;
  mov ebx, [ebp+GarbageReg]           ;
  mov ecx, eax                        ;
  call RndCall                        ;
  and eax, 01h                        ;
  test eax, eax                       ;
  jz Cg_Add                           ;
  call SubRegImm32                    ;
  ret                                 ;
Cg_Add:                               ;
  call AddRegImm32                    ;
  ret                                 ;
                                      ;
;=======================================================================================================
; Gnration d'un <add esp, 8>
;=======================================================================================================
Cg_AddEsp8_a:                         ;add esp, 8
  mov ax, 0C483h                      ;
  stosw                               ;
  mov al, 008h                        ;
  stosb                               ;
  ret                                 ;
Cg_AddEsp8_b:                         ;pop reg/pop reg
  mov ebx, [ebp+GarbageReg]           ;
  call PopReg                         ;
  mov ebx, [ebp+GarbageReg]           ;
  call PopReg                         ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <mov reg, imm32>
;=======================================================================================================
_MovRegImm32_:                        ;
  call RndCall                        ;
  and eax, 03h                        ;
  test eax, eax                       ;
  jz Cg_MovRegImm32_a                 ;
  cmp eax, 01h                        ;
  je Cg_MovRegImm32_b                 ;
  cmp eax, 02h                        ;
  je Cg_MovRegImm32_c                 ;
  jmp Cg_MovRegImm32_d                ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <push reg>
;=======================================================================================================
_PushReg_:                            ;
  call RndCall                        ;
  and eax, 01h                        ;
  test eax, eax                       ;
  jz Cg_PushReg_a                     ;
  jmp Cg_PushReg_b                    ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <dec reg>
;=======================================================================================================
_DecReg_:                             ;
  call RndCall                        ;
  and eax, 05h                        ;
  test eax, eax                       ;
  jz Cg_DecReg_a                      ;
  cmp eax, 01h                        ;
  je Cg_DecReg_b                      ;
  cmp eax, 02h                        ;
  je Cg_DecReg_c                      ;
  cmp eax, 03h                        ;
  je Cg_DecReg_d                      ;
  cmp eax, 04h                        ;
  je Cg_DecReg_e                      ;
  jmp Cg_DecReg_f                     ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <inc reg>
;=======================================================================================================
_IncReg_:                             ;
  call RndCall                        ;
  and eax, 05h                        ;
  test eax, eax                       ;
  jz Cg_IncReg_a                      ;
  cmp eax, 01h                        ;
  je Cg_IncReg_b                      ;
  cmp eax, 02h                        ;
  je Cg_IncReg_c                      ;
  cmp eax, 03h                        ;
  je Cg_IncReg_d                      ;
  cmp eax, 04h                        ;
  je Cg_IncReg_e                      ;
  jmp Cg_IncReg_f                     ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <test reg, reg>
;=======================================================================================================
_TestRegReg_:                         ;
  call RndCall                        ;
  and eax, 04h                        ;
  test eax, eax                       ;
  jz Cg_TestRegReg_a                  ;
  cmp eax, 01h                        ;
  je Cg_TestRegReg_b                  ;
  cmp eax, 02h                        ;
  je Cg_TestRegReg_c                  ;
  cmp eax, 03h                        ;
  je Cg_TestRegReg_d                  ;
  jmp Cg_TestRegReg_e                 ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <pop reg>
;=======================================================================================================
_PopReg_:                             ;
  call RndCall                        ;
  and eax, 01h                        ;
  test eax, eax                       ;
  jz Cg_PopReg_a                      ;
  jmp Cg_PopReg_b                     ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <Jmp Xxx>
;=======================================================================================================
_JmpXxx_:                             ;
  call RndCall                        ;
  and eax, 11h                        ;
  test eax, eax                       ;
  jz Cg_JmpXxx_a                      ;
  cmp eax, 01h                        ;
  je Cg_JmpXxx_b                      ;
  cmp eax, 02h                        ;
  je Cg_JmpXxx_c                      ;
  cmp eax, 03h                        ;
  je Cg_JmpXxx_d                      ;
  cmp eax, 04h                        ;
  je Cg_JmpXxx_e                      ;
  cmp eax, 05h                        ;
  je Cg_JmpXxx_f                      ;
  cmp eax, 06h                        ;
  je Cg_JmpXxx_g                      ;
  cmp eax, 07h                        ;
  je Cg_JmpXxx_h                      ;
  cmp eax, 08h                        ;
  je Cg_JmpXxx_i                      ;
  cmp eax, 09h                        ;
  je Cg_JmpXxx_j                      ;
  cmp eax, 0Ah                        ;
  je Cg_JmpXxx_k                      ;
  cmp eax, 0Bh                        ;
  je Cg_JmpXxx_l                      ;
  cmp eax, 0Ch                        ;
  je Cg_JmpXxx_m                      ;
  cmp eax, 0Dh                        ;
  je Cg_JmpXxx_n                      ;
  cmp eax, 0Eh                        ;
  je Cg_JmpXxx_o                      ;
  cmp eax, 0Fh                        ;
  je Cg_JmpXxx_p                      ;
  cmp eax, 10h                        ;
  je Cg_JmpXxx_q                      ;
  jmp Cg_JmpXxx_r                     ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour <add esp, 8>
;=======================================================================================================
_AddEsp8_:                            ;
  call RndCall                        ;
  and eax, 01h                        ;
  test eax, eax                       ;
  jz Cg_AddEsp8_a                     ;
  jmp Cg_AddEsp8_b                    ;
  ret                                 ;
                                      ;
;=======================================================================================================
; interface pour l'appel au gnrateur de nombres alatoires
;=======================================================================================================
RndCall:                                ;
  push dword ptr[ebp+iPrimeNumber2]     ;
  push dword ptr[ebp+iPrimeNumber1]     ;
  push dword ptr[ebp+piSeed]            ;
  call dword ptr[ebp+pRNG]              ;
  ret                                   ;
                                        ;
